// SPDX-License-Identifier: GPL-3.0-or-later
/*  kernelver.h -- get the version of a running Linux kernel

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

#ifndef _KERNELVER_H_
#define _KERNELVER_H_

#ifdef __cplusplus
extern "C"
{
#endif

  unsigned int linux_version (void);

#ifdef __cplusplus
}
#endif

#endif				/* _KERNELVER_H_ */
